/*-------------------------------------------------------------------------------
 * Project: KoreBot Library	
 * $Author: jtharin $
 * $Date: 2009-09-02 08:45:11 $
 * $Revision: 1.1.1.1 $
 * 
 * 
 * $Header: /mnt/home/cvs/PicoITXsoftware/libkorebot-1.14-kb1/src/kb_gpio.c,v 1.1.1.1 2009-09-02 08:45:11 jtharin Exp $
 */

/*! 
 * \file   kb_gpio.c Support for KoreBot General Purpose IO.             
 *
 * \brief 
 *         This module provides useful basic facilities to use the
 *         KoreBot GPIO and configure them.
 *
 * \author   Pierre Bureau (K-Team SA)
 * \modified  2010-02-08 J. Tharin: adapted for PicoITX
 *        
 *
 * \note     Copyright (C) 2004,2010 K-TEAM SA
 * \bug      none discovered.                                         
 * \todo     nothing.
 */
 

#include <sys/mman.h>
#include "korebot.h"


// uint def
#include <stdint.h>


// outn,...
#include <sys/io.h>


// pci_read_word,...
//#include <pci/pci.h>


// via gpio driver
#include "via_gpio.h"



/****************************************************************
*
* Defines
*
*/ 




/**********************************
*
* GPIO chip parameters
*
*/

// CX700/VX700 
#define VENDOR 0x1106

#define CLASS 0x0601

#define DEVICE_ID 0x8324


#define PCI_BUS 0x0

#define PCI_DEVICE 0x11

#define PCI_FUNCTION 0x0

//  PCI domain (host bridge)
#define PCI_DOMAIN 0x0 


/********************************
*
* GPIO
*
*/


#define MAX_IO 4

#define LOWEST_IO 2
#define HIGHEST_IO 5


// IO in input: 2,3,4,5
#define IO_INPUT_MASK  0b00111100

// IO in output: 2,3
#define IO_OUTPUT_MASK 0b00001100


/********************************
*
* other defines
*
*/



/********************************************************************
* 
* Global variables
*
*/


static int fd;


/********************************************************************
* 
* Internal functions
*
*/

static void bit_set(unsigned long addr,unsigned bit)
{
  

  t_gpio_port gpio_port;

	gpio_port.bit=bit;
  gpio_port.address=addr;
  ioctl(fd,GPIO_SET,&gpio_port);  
  
}

static void bit_clear(unsigned long addr,unsigned bit)
{
  t_gpio_port gpio_port;

	gpio_port.bit=bit;
  gpio_port.address=addr;
  ioctl(fd,GPIO_CLEAR,&gpio_port);  

  
}

static int bit_test(unsigned long addr,unsigned bit)
{
  t_gpio_port gpio_port;
  
  gpio_port.bit=bit;
  gpio_port.address=addr;
  
  
  ioctl(fd,GPIO_TEST,&gpio_port);
 	return  gpio_port.value;
    
}



/********************************************************************
* 
* Library functions
*
*/


/*!
 * Initialize the GPIO module. This function must be called before any
 * other call the gpio functions.
 *
 * \return 
 * 	- 0 on success
 *	- a negative error code on failure
 */
int kb_gpio_init()
{

  /* Open the  device */
	if ( (fd = open("/dev/via_gpio",O_RDWR)) == -1)
  {
    //fprintf(stderr, "\nERROR: cannot open /dev/via_gpio device! %d\n",fd);
		KB_ERROR("kb_gpio_init",KB_ERROR_FILEOPEN,"/dev/via_gpio");
    return -KB_ERROR_FILEOPEN;
  }
 


  return 0;
}

/*!
 * Cleanup the GPIO module. Must be called to free the module ressources.
 */
int kb_gpio_cleanup()
{
  close(fd);  
}



/*!
 * Configure the given GPIO pin function. Check first if the pin is freely
 * available on the KoreBot.
 *
 * \param gpio A gpio number between 0 and 84
 * \param function 	0:IO mode 
 * 			1:Alternate function 1 
 * 			2:Alternate function 2
 * 			3:Alternate function 3
 */
void kb_gpio_function(unsigned gpio, unsigned function)
{


 // not used for PicoITX

 /* if(gpio > MAXIO || function > 3)
    KB_ERROR("kb_gpio_get",KB_ERROR_INVALID);

  if((function & 0x1))
    bit_set(GAFR,2*gpio);
  else
    bit_clear(GAFR,2*gpio);

  if((function & 0x2))
    bit_set(GAFR,(2*gpio)+1);
  else
    bit_clear(GAFR,(2*gpio)+1);
    
    */
}

/*!
 * Configure the given GPIO as input or output. 
 * 
 * \param gpio A gpio number between 2 and 5
 * \param dir The direction 0:output 1:input
 */
void kb_gpio_dir(unsigned gpio, unsigned dir)
{
  if((gpio > HIGHEST_IO) || (gpio < LOWEST_IO))
  {
    KB_ERROR("kb_gpio_dir",KB_ERROR_INVALID,gpio);
    return;
  }
  
  if(dir)
  {
    // input 
    if(!(1<<gpio & IO_INPUT_MASK))
    {
      KB_ERROR("kb_gpio_dir",KB_ERROR_INVALID,dir);
      return;
    }
    
    
    if (gpio == 2) // GPIO 2 ?
    {    
      bit_set(0x4d,6);
      
    }
    else
    if (gpio == 3) // GPIO 3 ?
    {
      bit_set(0x4d,7); 
    }
        

  }
  else
  {
    // output
    if(!(1<<gpio & IO_OUTPUT_MASK))
    {
      KB_ERROR("kb_gpio_dir",KB_ERROR_INVALID,dir);
      return;
    }
    
    // nothing to be done
    
    
  }
  
  
}

/*! 
 * Set a given GPIO. The GPIO must be first configured to ouput mode.
 * If the GPIO is configured as input, it will be set when 
 * configured as ouput.
 *
 * \param gpio A gpio number between 2 and 3
 */
void kb_gpio_set(unsigned gpio)
{
 /* if(gpio > MAXIO)
    KB_ERROR("kb_gpio_get",KB_ERROR_INVALID);

  bit_set(GPSR,gpio);
  
  */
   if(!(1<<gpio & IO_OUTPUT_MASK))
      KB_ERROR("kb_gpio_set",KB_ERROR_INVALID,gpio);
  
  switch (gpio)
  {
    case 2:
        //	Output register: PMIO Rx4D[6]
        bit_set(0x4d,6);   
      break;
    case 3:
         //Output register: PMIO Rx4D[7]
         bit_set(0x4d,7);
        
      break;
    
    default:
      break;
  }
  
}

/*! 
 * Clear a given GPIO. The GPIO must be first configured to ouput mode.
 * If the GPIO is configured as input, it will be cleared when 
 * configured as ouput.
 *
 * \param gpio A gpio number between 2 and 3
 */
void kb_gpio_clear(unsigned gpio)
{
  
  if(!(1<<gpio & IO_OUTPUT_MASK))
    KB_ERROR("kb_gpio_clear",KB_ERROR_INVALID,gpio);
  
  switch (gpio)
  {
    case 2:
        //	Output register: PMIO Rx4D[6]
        bit_clear(0x4d,6);   
      break;
    case 3:
         //Output register: PMIO Rx4D[7]
         bit_clear(0x4d,7);      
      break;
    default:      
      break;
  }
}

/*!
 * Get the current level for a given GPIO. It can be used regardless 
 * of the current GPIO configuration.
 *
 * \param gpio A gpio number between 2 and 5
 *
 * \return 
 * 	- 1 if the GPIO is set
 * 	- 0 if the GPIO is cleared
 * 	- an error code if the GPIO is invalid
 */
int kb_gpio_get(unsigned gpio)
{

  int val;

  if(!(1<<gpio & IO_INPUT_MASK))
  {
    KB_ERROR("kb_gpio_get",KB_ERROR_INVALID,gpio);
    return -KB_ERROR_INVALID;
  }


  switch (gpio)
  {
    case 2:
        //  status/input register : PMIO Rx49[6]
        val=bit_test(0x49,6);        
  
      break;
    case 3:
        //  status/input register : PMIO Rx49[7]
        val=bit_test(0x49,7);        
   
      break;
    case 4:
        //  status/input register : PMIO Rx48[4]
        val=bit_test(0x48,4);        
  
      break;
    case 5:
        //  status/input register : PMIO Rx48[5]
        val=bit_test(0x48,5);        
   
      break;
    default:
      break;
  }


  return val;
}
